// DSoundLayer.cpp: Implementierung der Klasse DSoundLayer.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DSoundLayer.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

DSoundLayer::DSoundLayer()
{

	g_pUtils->Log("\nDirectSound initialization");

	// COM stuff
	CoInitialize(NULL);

	CoCreateInstance(CLSID_DirectMusicLoader, NULL, 
		CLSCTX_INPROC, IID_IDirectMusicLoader8,
		(void**)&m_pLoader);

	if (!m_pLoader) throw InitException("Unable to create DM Loader!");

	CoCreateInstance(CLSID_DirectMusicPerformance, NULL,
		CLSCTX_INPROC, IID_IDirectMusicPerformance8,
		(void**)&m_pPerformance );

	if (!m_pPerformance) throw InitException("Unable to create DM Performance!");


	// audio init
	HRESULT hres;

	hres = m_pPerformance->InitAudio( 
		NULL,      // IDirectMusic interface not needed.
		NULL,      
		g_hWnd,      // Window handle.
		DMUS_APATH_DYNAMIC_STEREO,
		64,        // Number of performance channels.
		DMUS_AUDIOF_ALL,   // Features on synthesizer.
		NULL       // Audio parameters; use defaults.
	);
	if (FAILED(hres)) throw InitException(hres, "InitAudio FAILED!");


	std::string mediaPath = g_pUtils->GetFullPath("media");

	WCHAR wstrSearchPath[128];
	MultiByteToWideChar( CP_ACP, 0, mediaPath.c_str(), -1, 
		 wstrSearchPath, 128 );

	// create AudioPath
	hres = m_pPerformance->CreateStandardAudioPath( 
		DMUS_APATH_DYNAMIC_3D,  // Path type.
		64,                     // Number of performance channels.
		TRUE,                   // Activate now.
		&m_p3DAudioPath           // Pointer that receives audiopath.
	);
	if (FAILED(hres)) throw InitException(hres, "CreateStandardAudioPath FAILED!");


	// get DirectSound3DBuffer interface
	hres = m_p3DAudioPath->GetObjectInPath( 
		DMUS_PCHANNEL_ALL,  // Performance channel.
		DMUS_PATH_BUFFER,   // Stage in the path.
		0,                  // Index of buffer in chain.
		GUID_NULL,          // Class of object.
		0,                  // Index of object in buffer; ignored.
		IID_IDirectSound3DBuffer, // GUID of desired interface.
		(LPVOID*) &m_pDSB     // Pointer that receives interface.
	);
	if (FAILED(hres)) throw InitException(hres, "GetObjectInPath FAILED!");


	// Set the search directory.
	hres = m_pLoader->SetSearchDirectory( 
		GUID_DirectMusicAllTypes, // Types of files sought.
		wstrSearchPath,     // Where to look.
		FALSE         // Don't clear object data.
	);
	if (FAILED(hres)) 
	{
		char msg[64];
		sprintf(msg, "SetSearchDirectory FAILED! (%s)", mediaPath.c_str());
		throw InitException(hres, msg);
	}




	g_pUtils->Log("DirectSound initialization finished\n");
}

DSoundLayer::~DSoundLayer()
{


	if (m_pPerformance)
	{
		m_pPerformance->Stop(
			NULL, // Stop all segments.
			NULL, // Stop all segment states.
			0,  // Do it immediately.
			0   // Flags.
		);

		m_pPerformance->CloseDown();
	}
	SAFE_RELEASE(m_pPerformance);
	SAFE_RELEASE(m_pLoader);

	for (unsigned i=0; i<m_vpSegment.size(); i++)
	{
		if (m_vpSegment[i]) m_vpSegment[i]->Release();
	}

	CoUninitialize();
}

handleSound DSoundLayer::CreateSound(const char *_filename)
{
	std::string fullname = g_pUtils->GetFullPath(_filename);

	WCHAR wstrFileName[128];
	MultiByteToWideChar( CP_ACP, 0, fullname.c_str(), -1, 
		 wstrFileName, 128 );

	
	m_vpSegment.push_back(NULL);
	int num = m_vpSegment.size()-1;

	HRESULT hres;
	hres = m_pLoader->LoadObjectFromFile(
		CLSID_DirectMusicSegment, // Class identifier.
		IID_IDirectMusicSegment8, // ID of desired interface.
		wstrFileName,     // Filename.
		(LPVOID*) &m_vpSegment[num]   // Pointer that receives interface.
	);
	if (FAILED(hres))
	{
		char msg[64];
		sprintf(msg, "Unable to create sound %s", fullname.c_str());
		throw InitException(hres, msg);
	}

	m_vpSegment[num]->Download( m_pPerformance );

	g_pUtils->Log(std::string("Created sound ") + _filename);


	return num;

}

void DSoundLayer::PlaySound(handleSound _sound)
{

	m_pPerformance->PlaySegmentEx(
		m_vpSegment[_sound],  // Segment to play.
		NULL,    // Used for songs; not implemented.
		NULL,    // For transitions. 
		0,     // Flags.
		0,     // Start time; 0 is immediate.
		NULL,    // Pointer that receives segment state.
		NULL,    // Object to stop.
		m_p3DAudioPath   // Audiopath, if not default.
	);  

}

void DSoundLayer::SetSoundPosition(D3DXVECTOR3 _pos)
{
	m_pDSB->SetPosition( _pos.x, _pos.y, _pos.z, DS3D_IMMEDIATE );
}
