// Profiler.cpp: Implementierung der Klasse Profiler.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Profiler.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

Profiler::Profiler()
{
	unsigned int i;

	for( i=0; i<NUM_PROFILE_SAMPLES; i++ )
	{
		m_Sample[i].bValid = false;
		m_History[i].bValid = false;
	}

	LARGE_INTEGER tmp;
	QueryPerformanceFrequency(&tmp);
	m_fTicksPerSecond = (float)tmp.QuadPart;

	QueryPerformanceCounter(&m_liProfileStart);
	m_liProfileStop.QuadPart = 0;
}

void Profiler::ProfileStart(char* _name)
{
	unsigned int i = 0;

	while( i < NUM_PROFILE_SAMPLES && m_Sample[i].bValid == true )
	{
		if( strcmp( m_Sample[i].szName, _name ) == 0 ) 
		{
			//Found the sample
			m_Sample[i].iOpenProfiles++;
			m_Sample[i].iProfileInstances++;
			QueryPerformanceCounter(&m_Sample[i].liStartTime);
			assert( m_Sample[i].iOpenProfiles == 1 ); //max 1 open at once
			return;
		}
		i++;	
	}

	if( i >= NUM_PROFILE_SAMPLES ) {
		assert( !"Exceeded Max Available Profile Samples" );
		return;
	}

	strcpy( m_Sample[i].szName, _name );
	m_Sample[i].bValid = true;
	m_Sample[i].iOpenProfiles = 1;
	m_Sample[i].iProfileInstances = 1;
	m_Sample[i].liAccumulator.QuadPart = 0;
	QueryPerformanceCounter(&m_Sample[i].liStartTime);
	m_Sample[i].liChildrenSampleTime.QuadPart = 0;
}

void Profiler::ProfileStop(char* _name)
{
	unsigned int i = 0;
	unsigned int numParents = 0;

	while( i < NUM_PROFILE_SAMPLES && m_Sample[i].bValid == true )
	{
		if( strcmp( m_Sample[i].szName, _name ) == 0 )
		{  //Found the sample
			unsigned int inner = 0;
			int parent = -1;
			LARGE_INTEGER liEndTime;
			QueryPerformanceCounter(&liEndTime);
			m_Sample[i].iOpenProfiles--;

			//Count all parents and find the immediate parent
			while( m_Sample[inner].bValid == true ) 
			{
				if( m_Sample[inner].iOpenProfiles > 0 )
				{  //Found a parent (any open profiles are parents)
					numParents++;
					if( parent < 0 )
					{  //Replace invalid parent (index)
					  parent = inner;
					}
					else if( m_Sample[inner].liStartTime.QuadPart >=
							m_Sample[parent].liStartTime.QuadPart )
					{  //Replace with more immediate parent
					  parent = inner;
					}
				}
				inner++;
			}

			//Remember the current number of parents of the sample
			m_Sample[i].iNumParents = numParents;

			if( parent >= 0 )
			{  //Record this time in fChildrenSampleTime (add it in)
				m_Sample[parent].liChildrenSampleTime.QuadPart += liEndTime.QuadPart - m_Sample[i].liStartTime.QuadPart;
			}

			//Save sample time in accumulator
			m_Sample[i].liAccumulator.QuadPart += liEndTime.QuadPart - m_Sample[i].liStartTime.QuadPart;
			return;
		}
		i++;	
	}
}

void Profiler::DumpToBuffer()
{
	unsigned int i = 0;

	QueryPerformanceCounter(&m_liProfileStop);
	m_liFrameTime.QuadPart = m_liProfileStop.QuadPart - m_liProfileStart.QuadPart;

	char szFps[16];
	static float fps = 30.0f;
		fps = fps * 0.99f + m_fTicksPerSecond / (float)m_liFrameTime.QuadPart * 0.01f;

	sprintf( szFps, "FPS : %3.1f\n", fps );
	m_strBuffer = szFps;


	m_strBuffer += "  Ave :   Min :   Max :   # : Profile Name\n" ;
	m_strBuffer += "--------------------------------------------\n" ;

	while( i < NUM_PROFILE_SAMPLES && m_Sample[i].bValid == true )
	{		
		unsigned int indent = 0;
		LARGE_INTEGER sampleTime;
		float percentTime, aveTime, minTime, maxTime;
		char line[256];
		char ave[16], min[16], max[16], num[16];
			
		if( m_Sample[i].iOpenProfiles < 0 )
		{
			assert( !"ProfileEnd() called without a ProfileBegin()" );	
		}
		else if( m_Sample[i].iOpenProfiles > 0 ) 
		{
			assert( !"ProfileBegin() called without a ProfileEnd()" );
		}

		sampleTime.QuadPart = m_Sample[i].liAccumulator.QuadPart - m_Sample[i].liChildrenSampleTime.QuadPart;
		percentTime = ( (float)sampleTime.QuadPart / (float)(m_liFrameTime.QuadPart ) ) * 100.0f;

		aveTime = minTime = maxTime = percentTime;

		//Add new measurement into the history and get the ave, min, and max
		StoreProfileInHistory( m_Sample[i].szName, percentTime );
		GetProfileFromHistory( m_Sample[i].szName, &aveTime, &minTime, &maxTime );

		//Format the data
		sprintf( ave, "%3.1f", aveTime );
		sprintf( min, "%3.1f", minTime );
		sprintf( max, "%3.1f", maxTime );
		sprintf( num, "%3d", m_Sample[i].iProfileInstances );

		sprintf(line,"%5s : %5s : %5s : %3s : ", ave, min, max, num);
		m_strBuffer += line;

		for( indent=0; indent<m_Sample[i].iNumParents; indent++ ) 
		{
			m_strBuffer += "   ";
		}
		m_strBuffer += m_Sample[i].szName;
		m_strBuffer += "\n";
		i++;
	}

	//Reset samples for next frame
	for( i=0; i<NUM_PROFILE_SAMPLES; i++ )
	{
		m_Sample[i].bValid = false;
	}
	QueryPerformanceCounter(&m_liProfileStart);
}


void Profiler::StoreProfileInHistory(char *_name, float _percent)
{
	unsigned int i = 0;
	float oldRatio;
	float newRatio = 0.8f * (float)m_liFrameTime.QuadPart / m_fTicksPerSecond;
	if( newRatio > 1.0f ) 
	{
		newRatio = 1.0f;
	}
	oldRatio = 1.0f - newRatio;

	while( i < NUM_PROFILE_SAMPLES && m_History[i].bValid == true )
	{
		if( strcmp( m_History[i].szName, _name ) == 0 )
		{  //Found the sample
			m_History[i].fAve = (m_History[i].fAve*oldRatio) + (_percent*newRatio);
			if( _percent < m_History[i].fMin ) 
			{
				m_History[i].fMin = _percent;
			}
			else 
			{
				m_History[i].fMin = (m_History[i].fMin*oldRatio) + (_percent*newRatio);
			}

			if( _percent > m_History[i].fMax ) 
			{
				m_History[i].fMax = _percent;
			}
			else 
			{
				m_History[i].fMax = (m_History[i].fMax*oldRatio) + (_percent*newRatio);
			}
			return;
		}
		i++;
	}

	if( i < NUM_PROFILE_SAMPLES )
	{  //Add to history
		strcpy( m_History[i].szName, _name );
		m_History[i].bValid = true;
		m_History[i].fAve = m_History[i].fMin = m_History[i].fMax = _percent;
	}
	else
	{
		assert( !"Exceeded Max Available Profile Samples!");
	}
}

void Profiler::GetProfileFromHistory(char *_name, float *_ave, float *_min, float *_max)
{
	unsigned int i = 0;
	while( i < NUM_PROFILE_SAMPLES && m_History[i].bValid == true )
	{
		if( strcmp( m_History[i].szName, _name ) == 0 )
		{  //Found the sample
			*_ave = m_History[i].fAve;
			*_min = m_History[i].fMin;
			*_max = m_History[i].fMax;
			return;
		}
		i++;
	}	
	*_ave = *_min = *_max = 0.0f;
}


int Profiler::GetFrameTime()
{
	return (int)m_liFrameTime.QuadPart;
}
