// Terrain.cpp: Implementierung der Klasse Terrain.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Terrain.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

Terrain::Terrain()
{
	SETPARAMETERFILE("TerrainParams");

	m_vLightDir = g_pGame->GetLightDir();

	m_hDetailTex = g_pD3DRenderLayer->CreateTexture(LOADSTRING("m_szDetailTexName", "media/detail.jpg").c_str());
	m_hGroundTex = g_pD3DRenderLayer->CreateTexture(LOADSTRING("m_szGroundTexName", "media/perlintex.jpg").c_str(),
		256, 256, 1, 0, D3DFMT_X8R8G8B8, D3DPOOL_MANAGED, D3DX_FILTER_NONE, D3DX_FILTER_NONE, 0, 
		NULL, NULL);

	m_fScaleWidth  = LOADFLOAT("m_fScaleWidth",      80.0f);
	m_fScaleHeight = LOADFLOAT("m_fScaleHeight", 10.0f);

	m_iMaxPixelError = LOADINT("m_iMaxPixelError", 30);

	m_iBlockSize = LOADINT("m_iBlockSize", 17);

	m_iNumMipMapLevels = Math::ld(m_iBlockSize-1);
	assert(m_iNumMipMapLevels > 0);

	m_pPrimitiveCount = new int[m_iNumMipMapLevels*16];
	m_phIB = new handleIB[m_iNumMipMapLevels*16];

	CreateIndexBuffers();

	m_pPatch = NULL;

	LoadHeightmap(LOADSTRING("m_szHeightmapName", "media/perlin.bmp").c_str());
}

Terrain::~Terrain()
{
	if (m_pPatch)
	{
		for (int i=0; i<m_iNumPatchesX * m_iNumPatchesZ; i++)
		{
				if(m_pPatch[i].pTerrainBlock) delete [] m_pPatch[i].pTerrainBlock;
				DeleteNode(m_pPatch[i].pNodeTop);
		}
		delete [] m_pPatch;
	}
	delete [] m_pPrimitiveCount;
	delete [] m_phIB;
}

void Terrain::LoadHeightmap(const char *_filename)
{

    BITMAPFILEHEADER bfh;
    BITMAPINFOHEADER bih;

	std::string fullname = g_pUtils->GetFullPath(_filename);

	// open file
    ifstream in(fullname.c_str(), ios_base::binary);
    if (in.fail())
    {
        std::string msg = "Unable to load heightfield bitmap! (";
        msg += _filename;
        msg += ")";
        throw InitException(msg);
    }

	// read headers
    in.read((char*)&bfh, sizeof(bfh));
    in.read((char*)&bih, sizeof(bih));
    if (bih.biCompression != 0)
    {
        std::string msg = "Bitmap compression not supported! (";
        msg += _filename;
        msg += ")";
        throw InitException(msg);
    }

	// check 8 bpp depth
    if (bih.biBitCount != 8)
    {
        std::string msg = "Only 8 bit heightfield supported! (";
        msg += _filename;
        msg += ")";
        throw InitException(msg);
    }

	m_iPatchWidth = 129;
	m_iPatchHeight = 129;


	// height&width
    if ((bih.biHeight-1) % (m_iPatchHeight-1) != 0 || (bih.biWidth-1) % (m_iPatchWidth-1) != 0)
    {
        std::string msg = "Heightmap must be 128*n+1 x 128*n+1! (";
        msg += _filename;
        msg += ")";
        throw InitException(msg);
    }

    // get first palette entry to set flipHeights if brighter colors are first
	int iFirstPalEntry;
	bool flipHeights = false;
	in.read((char*)&iFirstPalEntry, sizeof(int));
	if (iFirstPalEntry > 0x7FFFFF) flipHeights = true;

	// ignore rest of palette
    in.ignore(bfh.bfOffBits - sizeof(bfh) - sizeof(bih)- sizeof(int));


	// store bitmap dimensions, calculate tree dimensions
	m_iTotalWidth = bih.biWidth;
	m_iTotalHeight = bih.biHeight;
	m_iNumPatchesX = (m_iTotalWidth-1) / (m_iPatchWidth-1);
	m_iNumPatchesZ = (m_iTotalHeight-1) / (m_iPatchHeight-1);

	if((m_iPatchWidth-1)%(m_iBlockSize-1)!=0) throw InitException("Block Size must be 2^n+1!");

	m_iBlockRes = (m_iPatchWidth-1)/(m_iBlockSize-1);

	m_iQuadTreeDepth = 0;
	int b = m_iBlockSize-1;
	while ( (b<<=1) < m_iPatchWidth) m_iQuadTreeDepth++;

	// read bitmap into pHeightmap
	m_pHeightData = new BYTE[m_iTotalWidth * m_iTotalHeight];
	BYTE * pLoc = m_pHeightData;
	int border = m_iTotalWidth % 4;	// bitmap is DWORD-aligned
	if (border) border=4-border;

	for (int h = 0; h<m_iTotalHeight; h++)
	{
	    in.read((char*)pLoc, m_iTotalWidth);
		in.ignore(border);
		pLoc += m_iTotalWidth;
	}

	// create Vertices and fill with terrain data
	// begin at lower-left with z-axis upwards and x-axis to the right
	// This is not the final VB

	TERRAINVERTEX * pGrid = new TERRAINVERTEX[m_iTotalWidth * m_iTotalHeight];
	for (int z = 0; z<m_iTotalHeight; z++)
	{
		for (int x=0; x<m_iTotalWidth; x++)
		{
			int ind = x + m_iTotalWidth*z;
			if(flipHeights) m_pHeightData[ind] = 255 - m_pHeightData[ind];

			pGrid[ind].pos.x = m_fScaleWidth * (float)x;
			pGrid[ind].pos.z = m_fScaleWidth * (float)z;
			pGrid[ind].pos.y = m_fScaleHeight * m_pHeightData[ind]; 
			pGrid[ind].tu1 =        (float)x / (m_iTotalWidth -1);
			pGrid[ind].tv1 = (1.0f- (float)z / (m_iTotalHeight-1));
			pGrid[ind].tu2 =        (float)x / (m_iTotalWidth -1)  * LOADINT("m_iDetailTextureRes", 30);
			pGrid[ind].tv2 = (1.0f- (float)z / (m_iTotalHeight-1)) * LOADINT("m_iDetailTextureRes", 30);
		}
	}

	
	// calculate static lighting
	CalcStaticLighting(m_vLightDir, g_pGame->GetAmbient());

	m_pPatch = new TerrainPatch[m_iNumPatchesX * m_iNumPatchesZ];

	for (int p=0; p<m_iNumPatchesX * m_iNumPatchesZ; p++)
		BuildQuadTree(&m_pPatch[p], pGrid);

	delete [] pGrid;
}

void Terrain::Render(Camera _camera)
{

	D3DXMATRIX mat;
	D3DXMatrixIdentity(&mat);
	g_pD3DRenderLayer->SetTransform(D3DTS_WORLD, &mat);

	g_pD3DRenderLayer->SetVertexShader(D3DFVF_TERRAINVERTEX);
	g_pD3DRenderLayer->SetRenderState(D3DRS_LIGHTING, 0);


	g_pD3DRenderLayer->SetTexture(0, m_hGroundTex);
	g_pD3DRenderLayer->SetTexture(1, m_hDetailTex);


    g_pD3DRenderLayer->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0 );
    g_pD3DRenderLayer->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
    g_pD3DRenderLayer->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    g_pD3DRenderLayer->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );

	g_pD3DRenderLayer->SetTextureStageState(1, D3DTSS_COLOROP,   D3DTOP_MODULATE2X );
	g_pD3DRenderLayer->SetTextureStageState(1, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	g_pD3DRenderLayer->SetTextureStageState(1, D3DTSS_COLORARG2, D3DTA_CURRENT);
	

	for (int i=0; i<m_iNumPatchesX * m_iNumPatchesZ; i++)
	{
		ProcessNode(&m_pPatch[i], m_pPatch[i].pNodeTop, _camera, false);
	}

	for (i=0; i<m_iNumPatchesX * m_iNumPatchesZ; i++)
	{
		if (m_pPatch[i].pNodeTop->clipstat != FI_OUTSIDE)
		{
			g_pD3DRenderLayer->SetStreamSource(0, m_pPatch[i].hVB, D3DXGetFVFVertexSize(D3DFVF_TERRAINVERTEX));
			RenderNode(&m_pPatch[i], m_pPatch[i].pNodeTop, _camera, false);	
		}
	}

	g_pD3DRenderLayer->SetRenderState(D3DRS_LIGHTING, 1);

}











/**
 * Determines for each vertex if is reached by light and calculates a 
 * grayscale value for the diffuse color.
 * If occlusion occurs, the neighboring vertices are dimmed too to avoid aliasing.
 * 
 * If the vertex is not occluded, the vertex normal is averaged
 */
void Terrain::CalcStaticLighting(D3DXVECTOR3 _lightDir, float _ambient)
{
	int x, z;

	// for each vertex, determine if shadowed
	D3DXVECTOR3 vLightIncr;
	if (_lightDir.x > _lightDir.z) 
		vLightIncr = -_lightDir * ( m_fScaleWidth / _lightDir.x);
	else
		vLightIncr = -_lightDir * ( m_fScaleWidth / _lightDir.z);


	int * pShadowed = new int[m_iTotalWidth * m_iTotalHeight];
	memset(pShadowed, 0, sizeof(int)*m_iTotalWidth * m_iTotalHeight);
	D3DXVECTOR3 vCurRayPoint;
	for (z=0; z<m_iTotalHeight; z++)
	{
		for (x=0; x<m_iTotalWidth; x++)
		{
			vCurRayPoint.x = x*m_fScaleWidth;
			vCurRayPoint.y = GetHeightAtCoords((float)x, (float)z)*m_fScaleHeight;
			vCurRayPoint.z = z*m_fScaleWidth;

			vCurRayPoint += vLightIncr;
			while (1)
			{
				if (vCurRayPoint.x < 0 || vCurRayPoint.z < 0 ||
					vCurRayPoint.x >= m_fScaleWidth*m_iTotalWidth || vCurRayPoint.z >= m_fScaleWidth*m_iTotalHeight)
					break;

				if (vCurRayPoint.y > 255*m_fScaleHeight) break;

				float h = GetHeightAt(vCurRayPoint.x, vCurRayPoint.z);
				if (h >= vCurRayPoint.y)
				{
					pShadowed[x + z*m_iTotalWidth] += 1;
					
					if (x-1>0) pShadowed[x-1 +  z   *m_iTotalWidth] += 1;
					if (z-1>0) pShadowed[x   + (z-1)*m_iTotalWidth] += 1;
					if (x+1<m_iTotalWidth) pShadowed[x+1 + z*m_iTotalWidth] += 1;
					if (z+1<m_iTotalHeight) pShadowed[x + (z+1)*m_iTotalWidth] += 1;

					if (x-1>0 && z-1>0)			
						pShadowed[x-1 + (z-1)*m_iTotalWidth] += 1;

					if (z-1>0 && x+1<m_iTotalWidth) 
						pShadowed[x+1 + (z-1)*m_iTotalWidth] += 1;

					if (x+1<m_iTotalWidth && z+1<m_iTotalHeight)
						pShadowed[x+1 + (z+1)*m_iTotalWidth] += 1;

					if (z+1<m_iTotalHeight && x-1)
						pShadowed[x-1 + (z+1)*m_iTotalWidth] += 1;

					break;
				}

				vCurRayPoint += vLightIncr;
			}
				
		}
	}


	// calc shading for vertices not in shadow
	float * pBrightness = new float[m_iTotalWidth * m_iTotalHeight];
	memset (pBrightness, 0, sizeof(float)*m_iTotalWidth * m_iTotalHeight);
	for (z=0; z<m_iTotalHeight; z++)
	{
		for (x=0; x<m_iTotalWidth; x++)
		{
			if (pShadowed[x + z*m_iTotalWidth]) 
			{
				pBrightness[x + z*m_iTotalWidth] = (float)pShadowed[x + z*m_iTotalWidth] / 9.0f;
			} 

			D3DXVECTOR3 n = GetNormalAtCoords((float)x, (float)z);
			pBrightness[x + z*m_iTotalWidth] = Math::maxf(-D3DXVec3Dot(&_lightDir, &n), 0.0f);
		}
	}



	// lock texture & modulate brightness
	HRESULT hres;
	D3DLOCKED_RECT lr;
	int * pCurPixel;
	hres = g_pD3DRenderLayer->LockRect(m_hGroundTex, 0, &lr, NULL, 0 );
	if (FAILED(hres)) throw InitException(hres, "Unable to lock ground texture!");

	for (z=0; z<m_iTotalHeight-1; z++)
	{
		pCurPixel = (int*)lr.pBits + z* (lr.Pitch>>2);
		for (x=0; x<m_iTotalWidth-1; x++)
		{
			*pCurPixel = Math::ColorScale(*pCurPixel, pBrightness[x+m_iTotalWidth*(m_iTotalWidth-1-z)] + _ambient);

			pCurPixel++;
		}
	}

	g_pD3DRenderLayer->UnlockRect(m_hGroundTex, 0);


	delete [] pBrightness;
	delete [] pShadowed;
}

void Terrain::BuildQuadTree(TerrainPatch * _patch, TERRAINVERTEX * _pGrid)
{
	_patch->pTerrainBlock = new TerrainBlock[m_iBlockRes*m_iBlockRes];


	// total number of vertices for VB; block edge vertices are duplicated
	static int numVertices = m_iBlockRes*m_iBlockRes * (int)pow(m_iBlockSize, 2);
	TERRAINVERTEX * pTV = new TERRAINVERTEX[numVertices];


	_patch->pNodeTop = CreateQuadNode(_patch, 0, 0, pTV, _pGrid);

	// Fill VB
	_patch->hVB = g_pD3DRenderLayer->CreateVertexBuffer(numVertices, sizeof(TERRAINVERTEX), D3DUSAGE_WRITEONLY, D3DFVF_TERRAINVERTEX);
	g_pD3DRenderLayer->FillVB(_patch->hVB, (BYTE*)pTV, numVertices, sizeof(TERRAINVERTEX));


	delete [] pTV;
}

/**
 * Fills _pTV with vertices from _pGrid and builds quadtree structure.
 * pTV are vertices for each terrainblock in sequence (borders are duplicated).
 * pGrid are all vertices from bottomleft to topright.
 * _number is current block number, starts from 0 for every patch, from bottom left to upper right
 */
QuadNode * Terrain::CreateQuadNode(TerrainPatch * _patch, int _depth, int _number, TERRAINVERTEX * _pTV, TERRAINVERTEX * _pGrid)
{

	QuadNode * pNode = new QuadNode();

	// leaf reached
	if (_depth == m_iQuadTreeDepth)
	{
		// get coordinates of current terrain patch
		int patchX = (_patch - m_pPatch) % m_iNumPatchesX;
		int patchZ = (_patch - m_pPatch) / m_iNumPatchesX;

		// calculate offset into _pTV
		int iVBOffset = _number * m_iBlockSize*m_iBlockSize;

		// coordinates in patch
		int x = _number % m_iBlockRes;
		int z = _number / m_iBlockRes;

		// offset in patch
		int iGridOffset = (m_iBlockSize-1)*x + m_iTotalWidth*(m_iBlockSize-1)*z;
		// add left and lower patches to offset to get total offset
		iGridOffset += patchX * (m_iPatchWidth-1) + m_iTotalWidth * patchZ * (m_iPatchHeight-1);


		// null out son pointers
		pNode->pLL = pNode->pLR = pNode->pTL = pNode->pTR = NULL;

		// set block pointer
		pNode->pBlock = &_patch->pTerrainBlock[_number];

		// set triangle index offset
		pNode->pBlock->iBaseVertexIndex = iVBOffset;

		pNode->pBlock->iMipMapLevel = 0;

		// copy vertices into pTV
		for (int row=0; row<m_iBlockSize; row++)
		{
			memcpy(_pTV   + iVBOffset   + row*m_iBlockSize,
				   _pGrid + iGridOffset + row*m_iTotalWidth,
				   m_iBlockSize*sizeof(TERRAINVERTEX));
		}

		// get min&max height
		float minY, maxY;
		for (int i=iVBOffset; i<iVBOffset + m_iBlockSize*m_iBlockSize; i++)
		{
			if (i==iVBOffset || _pTV[i].pos.y < minY) minY = _pTV[i].pos.y;
			if (i==iVBOffset || _pTV[i].pos.y > maxY) maxY = _pTV[i].pos.y;
		}
		pNode->bb.minY = minY;
		pNode->bb.maxY = maxY;
		pNode->bb.minX = ((iGridOffset % m_iTotalWidth)) * m_fScaleWidth;
		pNode->bb.maxX = pNode->bb.minX + m_iBlockSize * m_fScaleWidth;
		pNode->bb.minZ = ((iGridOffset / m_iTotalWidth)) * m_fScaleWidth;
		pNode->bb.maxZ = pNode->bb.minZ + m_iBlockSize * m_fScaleWidth;

		// get min&max delta height
		CalcMaxDeltaY(pNode->pBlock, _pTV+iVBOffset);
	}
	else
	{
		pNode->pBlock = NULL;

		int col = _number % (1<<_depth);
		int row = _number / (1<<_depth);
		int baseNumber = 2*col + (4<<_depth)*row;

		pNode->pLL = CreateQuadNode(_patch, _depth+1, baseNumber,                 _pTV, _pGrid);
		pNode->pLR = CreateQuadNode(_patch, _depth+1, baseNumber +1,			  _pTV, _pGrid);
		pNode->pTL = CreateQuadNode(_patch, _depth+1, baseNumber + (2<<_depth),   _pTV, _pGrid);
		pNode->pTR = CreateQuadNode(_patch, _depth+1, baseNumber + (2<<_depth)+1, _pTV, _pGrid);

		// calc bounding box
		pNode->bb.maxX = Math::GetMax(pNode->pLL->bb.maxX,pNode->pLR->bb.maxX,
								      pNode->pTL->bb.maxX,pNode->pTR->bb.maxX);
		pNode->bb.minX = Math::GetMin(pNode->pLL->bb.minX,pNode->pLR->bb.minX,
									  pNode->pTL->bb.minX,pNode->pTR->bb.minX);

		pNode->bb.maxY = Math::GetMax(pNode->pLL->bb.maxY,pNode->pLR->bb.maxY,
									  pNode->pTL->bb.maxY,pNode->pTR->bb.maxY);
		pNode->bb.minY = Math::GetMin(pNode->pLL->bb.minY,pNode->pLR->bb.minY,
									  pNode->pTL->bb.minY,pNode->pTR->bb.minY);

		pNode->bb.maxZ = Math::GetMax(pNode->pLL->bb.maxZ,pNode->pLR->bb.maxZ,
									  pNode->pTL->bb.maxZ,pNode->pTR->bb.maxZ);
		pNode->bb.minZ = Math::GetMin(pNode->pLL->bb.minZ,pNode->pLR->bb.minZ,
									  pNode->pTL->bb.minZ,pNode->pTR->bb.minZ);
	}



	return pNode;
}


/**
 * Every possible combination of full/interleaved border for every
 * mipmaplevel gets an own IB.
 */
void Terrain::CreateIndexBuffers()
{
	int row,col,lvl,code;
	int blocksize = m_iBlockSize;

	// buffer larger than ever needed
	WORD * pIB = new WORD[blocksize*blocksize*2];
	WORD * pCurInd;

	for (lvl=0; lvl<m_iNumMipMapLevels; lvl++)
	{
		int stride = 1<<lvl;	// stride between vertices
		int res = (blocksize-1) / stride; //  vert & horz resolution of current mipmap level

		// LSB : left border has neighbor with higher mipmap-lvl, then R, T, L
		for (code =0; code<16; code++)
		{
			pCurInd = pIB;

			// left border interleaved
			if (code & 1)
			{
				*pCurInd++ = 0;
				*pCurInd++ = 0;
				*pCurInd++ =   stride*blocksize + stride;
				*pCurInd++ = 2*stride*blocksize;
				for(row=2*stride; row<blocksize-1; row+=2*stride)
				{
					*pCurInd++ =  row          *blocksize + stride;
					*pCurInd++ =  row          *blocksize;
					*pCurInd++ = (row+  stride)*blocksize + stride;
					*pCurInd++ = (row+2*stride)*blocksize;
				}
			}
			// left border full
			else
			{
				*pCurInd++ = 0;
				*pCurInd++ = stride*blocksize;
				for(row=stride; row<blocksize-1; row+=stride)
				{
					*pCurInd++ =  row        *blocksize+stride;
					*pCurInd++ = (row+stride)*blocksize;
				}
			}

			// top border interleaved
			if (code & 4)
			{
				*pCurInd++ = blocksize*(blocksize-1);
				*pCurInd++ = blocksize*(blocksize-1);
				*pCurInd++ = blocksize*(blocksize-1) - stride*blocksize +   stride;
				*pCurInd++ = blocksize*(blocksize-1)			   	    + 2*stride;
				for(col=2*stride; col<blocksize-1; col +=2*stride)
				{
					*pCurInd++ = blocksize*(blocksize-1) - stride*blocksize + col;
					*pCurInd++ = blocksize*(blocksize-1)				    + col;
					*pCurInd++ = blocksize*(blocksize-1) - stride*blocksize + col+  stride;
					*pCurInd++ = blocksize*(blocksize-1)				    + col+2*stride;
				}
			}
			// top border full
			else
			{
				*pCurInd++ = blocksize*(blocksize-1);
				*pCurInd++ = blocksize*(blocksize-1) + stride;
				for(col=stride; col<blocksize-1; col+=stride)
				{
					*pCurInd++ = blocksize*(blocksize-1) - stride*blocksize + col;
					*pCurInd++ = blocksize*(blocksize-1)				    + col+stride;
				}
			}
			
			// right border interleaved
			if (code & 2)
			{
				*pCurInd++ = blocksize*blocksize-1;
				*pCurInd++ = blocksize*blocksize-1;
				*pCurInd++ = blocksize*blocksize-1 -   stride*blocksize - stride;
				*pCurInd++ = blocksize*blocksize-1 - 2*stride*blocksize;
				for(row=2*stride; row<blocksize-1; row+=2*stride)
				{
					*pCurInd++ = blocksize*blocksize-1 -  row          *blocksize - stride;
					*pCurInd++ = blocksize*blocksize-1 -  row          *blocksize;
					*pCurInd++ = blocksize*blocksize-1 - (row+  stride)*blocksize - stride;
					*pCurInd++ = blocksize*blocksize-1 - (row+2*stride)*blocksize;
				}
			}
			// right border full
			else
			{
				*pCurInd++ = blocksize*blocksize-1;
				*pCurInd++ = blocksize*blocksize-1 - stride*blocksize;
				for(row=stride; row<blocksize-1; row+=stride)
				{
					*pCurInd++ = blocksize*blocksize-1 -  row        *blocksize - stride;
					*pCurInd++ = blocksize*blocksize-1 - (row+stride)*blocksize;
				}
			}

			// lower border interleaved
			if (code & 8)
			{
				*pCurInd++ = blocksize-1;
				*pCurInd++ = blocksize-1;
				*pCurInd++ = blocksize-1 + stride*blocksize -   stride;
				*pCurInd++ = blocksize-1                    - 2*stride;
				for(col=2*stride; col<blocksize-1; col+=2*stride)
				{
					*pCurInd++ = blocksize-1 + stride*blocksize -  col;
					*pCurInd++ = blocksize-1				    -  col;
					*pCurInd++ = blocksize-1 + stride*blocksize - (col+  stride);
					*pCurInd++ = blocksize-1  				    - (col+2*stride);
				}		
			}
			// lower border full
			else
			{
				*pCurInd++ = blocksize-1;
				*pCurInd++ = blocksize-1 - stride;
				for(col=stride; col<blocksize-1; col+=stride)
				{
					*pCurInd++ = blocksize-1 + stride*blocksize -  col;
					*pCurInd++ = blocksize-1 				    - (col+stride);
				}
			}


			// center area
			for (col=stride; col<blocksize-1-stride; col+=stride)
			{
				if (col%(2*stride))
				{
					*pCurInd++ = stride*blocksize + col;
					for(row=stride; row<blocksize-1; row+=stride)
					{
						*pCurInd++ = row*blocksize + col;
						*pCurInd++ = row*blocksize + col+stride;
					}
				}
				else
				{
					*pCurInd++ = blocksize*(blocksize-1) - stride*blocksize + col;
					for(row=stride; row<blocksize-1; row+=stride)
					{
						*pCurInd++ = blocksize*(blocksize-1) - row*blocksize + col;
						*pCurInd++ = blocksize*(blocksize-1) - row*blocksize + col+stride;
					}
				}
			}
			// store primitivecount for current IB
			m_pPrimitiveCount[lvl*16+code] = pCurInd - pIB - 2;

			// create IB for current patch
			m_phIB[lvl*16+code] = g_pD3DRenderLayer->CreateIndexBuffer(pCurInd - pIB, D3DUSAGE_WRITEONLY);
			g_pD3DRenderLayer->FillIB(m_phIB[lvl*16+code], pIB, pCurInd - pIB);

		}	// code
	}	// lvl

	delete [] pIB;
}

void Terrain::ProcessNode(TerrainPatch * _patch, QuadNode *_node, Camera _camera, bool _inside)
{
	// if not guaranteed to be inside frustrum perform culling
	if (!_inside)
	{
		_node->clipstat = _camera.FrustrumIntersect(&_node->bb);
		if (_node->clipstat == FI_OUTSIDE) return;
		else if (_node->clipstat == FI_INSIDE) _inside = true;
	}

	if (_node->pBlock)
	{
		// calc appropriate mipmap level
		static const float fac = (float)pow((g_pD3DLayer->GetRes().y)/(2*m_iMaxPixelError*_camera.m_fAspectHeight) *
							m_fScaleWidth/m_fScaleHeight,2);

		float dist = (_camera.m_vPos.x - _node->bb.minX)*(_camera.m_vPos.x - _node->bb.maxX) +
					 (_camera.m_vPos.y - _node->bb.minY)*(_camera.m_vPos.y - _node->bb.maxY) +
					 (_camera.m_vPos.z - _node->bb.minZ)*(_camera.m_vPos.z - _node->bb.maxZ);

		int mmlvl = 0;
		while (mmlvl < m_iNumMipMapLevels-1 &&
			   dist > fac*_node->pBlock->pMaxDeltaY[mmlvl]) mmlvl++;
		_node->pBlock->iMipMapLevel = mmlvl;


		ProcessBlock(_patch, _node->pBlock);

	} 
	else
	{
		ProcessNode(_patch, _node->pLL, _camera, _inside);
		ProcessNode(_patch, _node->pLR, _camera, _inside);
		ProcessNode(_patch, _node->pTL, _camera, _inside);
		ProcessNode(_patch, _node->pTR, _camera, _inside);	
	}
}

void Terrain::RenderNode(TerrainPatch * _patch, QuadNode *_node, Camera _camera, bool _inside)
{
	if (!_inside)
	{
		if (_node->clipstat == FI_OUTSIDE) return;
		else if (_node->clipstat == FI_INSIDE) _inside = true;
	}

	if (_node->pBlock)
	{
		RenderBlock(_patch, _node->pBlock);
	} 
	else
	{
		RenderNode(_patch, _node->pLL, _camera, _inside);
		RenderNode(_patch, _node->pLR, _camera, _inside);
		RenderNode(_patch, _node->pTL, _camera, _inside);
		RenderNode(_patch, _node->pTR, _camera, _inside);		
	}
}

void Terrain::RenderBlock(TerrainPatch * _patch, TerrainBlock *_block)
{
	assert(_block->iBaseVertexIndex % m_iBlockSize == 0);
	assert(_block->iMipMapLevel < m_iNumMipMapLevels);
	assert(_block->iMipMapLevel >= 0);


	unsigned char code = 0;
	TerrainPatch * neighborPatch;
	TerrainBlock * neighborBlock;


	//left
	neighborPatch = _patch;
	neighborBlock = _block;
	GetNeighbor(0, &neighborPatch, &neighborBlock);
	if (neighborBlock &&
		neighborBlock->iMipMapLevel - _block->iMipMapLevel > 0)
		code |= 1;

	// right
	neighborPatch = _patch;
	neighborBlock = _block;
	GetNeighbor(1, &neighborPatch, &neighborBlock);
	if (neighborBlock &&
		neighborBlock->iMipMapLevel - _block->iMipMapLevel > 0)
		code |= 2;

	// check upper border
	neighborPatch = _patch;
	neighborBlock = _block;
	GetNeighbor(2, &neighborPatch, &neighborBlock);
	if (neighborBlock &&
		neighborBlock->iMipMapLevel - _block->iMipMapLevel > 0)
		code |= 4;

	// check lower border
	neighborPatch = _patch;
	neighborBlock = _block;
	GetNeighbor(3, &neighborPatch, &neighborBlock);
	if (neighborBlock &&
		neighborBlock->iMipMapLevel - _block->iMipMapLevel > 0)
		code |= 8;

	
	g_pD3DRenderLayer->SetIndices(m_phIB[(_block->iMipMapLevel<<4)+code], _block->iBaseVertexIndex);
	g_pD3DRenderLayer->DrawIndexedPrimitive(D3DPT_TRIANGLESTRIP, 0, m_iBlockSize*m_iBlockSize, 0, m_pPrimitiveCount[(_block->iMipMapLevel<<4)+code]);
}

void Terrain::CalcMaxDeltaY(TerrainBlock *_block, TERRAINVERTEX *_pTV)
{
	_block->pMaxDeltaY = new float[m_iNumMipMapLevels-1];

	float maxDeltaY=0;
	float curY;
	float avY;

	for(int lvl =0; lvl<m_iNumMipMapLevels-1; lvl++)
	{
		int stride = 1<<lvl;

		for(int row=0; row<m_iBlockSize;row+=stride)
		{
			for (int col=0; col<m_iBlockSize; col+=stride)
			{
				if (row%(stride<<1)==0 && col%(stride<<1)==0) continue;

				// interpolate left&right neighbor
				if (row%(stride<<1)==0)
				{
					avY= (_pTV[col-stride + row*m_iBlockSize].pos.y + 
						  _pTV[col+stride + row*m_iBlockSize].pos.y) / 2;
				}
				// interpolate lower&upper neighbor
				else if(col%(stride<<1)==0)
				{
					avY= (_pTV[col + (row+stride)*m_iBlockSize].pos.y + 
						  _pTV[col + (row-stride)*m_iBlockSize].pos.y) / 2;
				}
				// interpolate all four neighbors (we don't know which diagonal will be chosen)
				else
				{
					avY= (_pTV[col-stride + row         *m_iBlockSize].pos.y + 
						  _pTV[col+stride + row         *m_iBlockSize].pos.y +
					      _pTV[col        + (row+stride)*m_iBlockSize].pos.y + 
						  _pTV[col        + (row-stride)*m_iBlockSize].pos.y) / 4;
				}

				curY = _pTV[col + row*m_iBlockSize].pos.y;

				if (Math::absf(curY - avY) > maxDeltaY) maxDeltaY = Math::absf(curY - avY);
			}
		}

		_block->pMaxDeltaY[lvl] = maxDeltaY*maxDeltaY;
	}
}

// check lower and left border to ensure mipmaplevel difference <=1, 
void Terrain::ProcessBlock(TerrainPatch * _patch, TerrainBlock *_block)
{
	int ind = _block - _patch->pTerrainBlock;
	int lvldiff;
	TerrainPatch * neighborPatch;
	TerrainBlock * neighborBlock;


	// left border
	neighborPatch = _patch;
	neighborBlock = _block;
	GetNeighbor(0, &neighborPatch, &neighborBlock);
	if (neighborBlock)
	{
		lvldiff = neighborBlock->iMipMapLevel - _block->iMipMapLevel;
		if (!lvldiff); //do nothing
		else if (lvldiff >1)
		{
			neighborBlock->iMipMapLevel = 1+_block->iMipMapLevel;
			ProcessBlock(neighborPatch, neighborBlock);
		}
		else if(lvldiff <-1) 
			_block->iMipMapLevel = 1+neighborBlock->iMipMapLevel;
	}

	// check lower border
	neighborPatch = _patch;
	neighborBlock = _block;
	GetNeighbor(3, &neighborPatch, &neighborBlock);
	if (neighborBlock)
	{
		lvldiff = neighborBlock->iMipMapLevel - _block->iMipMapLevel;
		if (!lvldiff);	// do nothing
		else if (lvldiff >1)
		{
			neighborBlock->iMipMapLevel = 1+ _block->iMipMapLevel;
			ProcessBlock(neighborPatch, neighborBlock);
		}
		else if (lvldiff <-1)
			_block->iMipMapLevel = 1+ neighborBlock->iMipMapLevel;
	}
}

/**
 *	Calculates the surface normal and the distance pos-surface if pos is below terrain.
 *
 *	Each Quad is split from LL to TR. If the position is below the tri, offset and normal
 *  for a perpendicular projection onto the triangle plane are calculated.
 *  As always, y is up, z is 'top', x is 'right'
 *
 */
bool Terrain::Collide(D3DXVECTOR3 const & _pos, float * _offset, D3DXVECTOR3 *_normal)
{
	assert(_offset);
	assert(_normal);

	// handle borders
	if(_pos.x<0)
	{
		if(_normal) 
			*_normal = D3DXVECTOR3(1.0f, 0.0f, 0.0f);
		if (_offset)
			*_offset = -_pos.x;

		return true;
	}
	if(_pos.z<0)
	{
		if(_normal) 
			*_normal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
		if (_offset)
			*_offset = -_pos.z;

		return true;
	}
	if(_pos.x>(m_iTotalWidth-1)*m_fScaleWidth) 
	{
		if(_normal)
			*_normal = D3DXVECTOR3(-1.0f, 0.0f, 0.0f);
		if (_offset)
			*_offset = _pos.x-(m_iTotalWidth-1)*m_fScaleWidth;

		return true;
	}
	if(_pos.z>(m_iTotalHeight-1)*m_fScaleWidth) 
	{
		if(_normal)
			*_normal = D3DXVECTOR3(0.0f, 0.0f, -1.0f);
		if (_offset)
			*_offset = _pos.z-(m_iTotalHeight-1)*m_fScaleWidth;

		return true;
	}

	// get coordinates of position
	float coordX, coordZ;
	coordX = _pos.x / m_fScaleWidth;
	coordZ = _pos.z / m_fScaleWidth;


	float terrainHeight = GetHeightAtCoords(coordX, coordZ)*m_fScaleHeight;

	if (_pos.y > terrainHeight) return false;

	D3DXVECTOR3 n = GetNormalAtCoords(coordX, coordZ);
	// vector from one point in triangle plane to pos
	D3DXVECTOR3 toPos = D3DXVECTOR3(_pos.x - (int)coordX*m_fScaleWidth,
									_pos.y - m_pHeightData[(int)coordX + (int)coordZ*m_iTotalWidth]*m_fScaleHeight,
									_pos.z - (int)coordZ*m_fScaleWidth);



	float ndist = D3DXVec3Dot(&toPos, &n);

	if (ndist < 0)
	{
		// project pos into triangle plane
		*_offset = -ndist;

		// store normal in provided pointer
		*_normal = n;

		return true;
	}

	return false;
}

/**
 *	Uses the result from the overloaded function to actually do the projection
 */
bool Terrain::Collide(D3DXVECTOR3 *_pos, D3DXVECTOR3*_n)
{
	assert (_pos);

	float offset;
	D3DXVECTOR3 v;

	if(!_n) _n = &v;

	bool c = Collide(*_pos, &offset, _n);
	if (c) *_pos += offset * *_n;

	return c;
}


/**
 *
 * gets neighboring terrain patch;
 * dir: 0 left, 1 right, 2 upper, 3 lower
 */
void Terrain::GetNeighbor(int _dir, TerrainPatch **_patch, TerrainBlock **_block)
{
	int patchInd = *_patch - m_pPatch;
	int blockInd = *_block - (*_patch)->pTerrainBlock;

	switch (_dir)
	{
		case 0:	// left
			if (blockInd % m_iBlockRes != 0)
			{
				(*_block)--;
				return;
			}
			if (patchInd % m_iNumPatchesX != 0)
			{
				(*_patch)--;
				*_block = &((*_patch)->pTerrainBlock[blockInd+ m_iBlockRes-1]);
				return;
			}
			*_block = NULL;
			*_patch = NULL;
			return;
		case 1:	// right
			if ((blockInd+1) % m_iBlockRes != 0)
			{
				(*_block)++;
				return;
			}
			if ((patchInd+1) % m_iNumPatchesX != 0)
			{
				(*_patch)++;
				*_block = &((*_patch)->pTerrainBlock[blockInd- m_iBlockRes+1]);
				return;
			}
			*_block = NULL;
			*_patch = NULL;
			return;	
		case 2:	// upper
			if (blockInd < m_iBlockRes*(m_iBlockRes-1))
			{
				(*_block) += m_iBlockRes;
				return;
			}
			if (patchInd < m_iNumPatchesX*(m_iNumPatchesZ-1))
			{
				(*_patch) += m_iNumPatchesX;		
				*_block = &((*_patch)->pTerrainBlock[blockInd- m_iBlockRes*(m_iBlockRes-1)]);
				return;
			}
			*_block = NULL;
			*_patch = NULL;
			return;
		case 3:	// lower
			if (blockInd >= m_iBlockRes)
			{
				(*_block) -= m_iBlockRes;
				return;
			}
			if (patchInd >= m_iNumPatchesX)
			{
				(*_patch) -= m_iNumPatchesX;		
				*_block = &((*_patch)->pTerrainBlock[blockInd+ m_iBlockRes*(m_iBlockRes-1)]);
				return;
			}
			*_block = NULL;
			*_patch = NULL;
			return;
		default:
			assert(!"Wrong dir specified in GetNeighbor");
			break;
	}
}

void Terrain::DeleteNode(QuadNode *_node)
{
	if (!_node) return;

	DeleteNode(_node->pLL);
	DeleteNode(_node->pLR);
	DeleteNode(_node->pTL);
	DeleteNode(_node->pTR);

	delete _node;
}

/**
 *	Returns the unscaled terrain height in the range 0-255 at the given terrain grid coordinate pair
 *	(linear interpolation)
 */
float Terrain::GetHeightAtCoords(float _x, float _z)
{
	if(_x <0 || _x >= m_iTotalWidth-1 || 
	   _z <0 || _z >= m_iTotalHeight-1) return 0.0f;

	int xInt = (int)(_x);
	int zInt = (int)(_z);
	float dx, dz;
	int a, b, c;

	if ((float)xInt == _x && (float)zInt == _z) return m_pHeightData[xInt + m_iTotalWidth*zInt];

	dx = _x - xInt;
	dz = _z - zInt;

	// lower right triangle
	if (dx > dz)
	{
		a = m_pHeightData[xInt+1 +  zInt   * m_iTotalWidth];
		b = m_pHeightData[xInt   +  zInt   * m_iTotalWidth];
		c = m_pHeightData[xInt+1 + (zInt+1)* m_iTotalWidth];
		return a + (1-dx)*(b-a) + dz*(c-a);
	}
	// top left triangle
	else
	{
		a = m_pHeightData[xInt   + (zInt+1)* m_iTotalWidth];
		b = m_pHeightData[xInt+1 + (zInt+1)* m_iTotalWidth];
		c = m_pHeightData[xInt   +  zInt   * m_iTotalWidth];
		return a + dx*(b-a) + (1-dz)*(c-a);
	}
}

D3DXVECTOR3 Terrain::GetNormalAtCoords(float _x, float _z, bool _normalize)
{
	int xInt = (int)(_x);
	int zInt = (int)(_z);
	D3DXVECTOR3 ret, a, b;
	float dx, dz;

	if((float)xInt == _x && (float)zInt == _z)
	{
		ret = GetNormalAtCoords(_x+0.1f, _z, 0) +
			  GetNormalAtCoords(_x-0.1f, _z, 0) +
			  GetNormalAtCoords(_x, _z+0.1f, 0) +
			  GetNormalAtCoords(_x, _z-0.1f, 0) /4;
		if (_normalize) D3DXVec3Normalize(&ret, &ret);
		return ret;
	}

	if ((float)xInt == _x)
	{
		ret = GetNormalAtCoords(_x+0.1f, _z, 0) +
			  GetNormalAtCoords(_x-0.1f, _z, 0) / 2;
		if (_normalize) D3DXVec3Normalize(&ret, &ret);
		return ret;
	}

	if ((float)zInt == _z)
	{
		return GetNormalAtCoords(_x, _z+0.1f) +
			   GetNormalAtCoords(_x, _z-0.1f) / 2;
	}

	if (_x < 0.0f || _z < 0.0f || _x >= (float)m_iTotalWidth-1 || _z >= (float)m_iTotalHeight-1) 
	{
		return D3DXVECTOR3(0.0f, 1.0f, 0.0f);
	}

	dx = _x - xInt;
	dz = _z - zInt;


	// lower right triangle
	if (dx>dz)
	{
		a.x = -m_fScaleWidth;
		a.y = (m_pHeightData[xInt   +  zInt   * m_iTotalWidth] -
			   m_pHeightData[xInt+1 +  zInt   * m_iTotalWidth]) * m_fScaleHeight;
		a.z =  0.0f;

		b.x =  0.0f;
		b.y = (m_pHeightData[xInt+1 + (zInt+1)* m_iTotalWidth] -
			   m_pHeightData[xInt+1 +  zInt   * m_iTotalWidth]) * m_fScaleHeight;
		b.z =  m_fScaleWidth;
	}
	// upper left triangle
	else
	{
		a.x =  m_fScaleWidth;
		a.y = (m_pHeightData[xInt+1 + (zInt+1)* m_iTotalWidth] -
			   m_pHeightData[xInt   + (zInt+1)* m_iTotalWidth]) * m_fScaleHeight;
		a.z =  0.0f;

		b.x =  0.0f;
		b.y = (m_pHeightData[xInt   +  zInt   * m_iTotalWidth] -
			   m_pHeightData[xInt   + (zInt+1)* m_iTotalWidth]) * m_fScaleHeight;
		b.z = -m_fScaleWidth;
	}

	D3DXVec3Cross(&ret, &a, &b);

	if (_normalize) D3DXVec3Normalize(&ret, &ret);

	return ret;
}


/**
 *  Returns the terrain height in world space units at a given world space coordinate pair
 */
float Terrain::GetHeightAt(float _x, float _z)
{
	return m_fScaleHeight*GetHeightAtCoords(_x / m_fScaleWidth, _z / m_fScaleWidth);
}

D3DXVECTOR3 Terrain::GetNormalAt(float _x, float _z, bool _normalize)
{
	return GetNormalAtCoords(_x / m_fScaleWidth, _z / m_fScaleWidth, _normalize);
}
